@@ -20,17 +20,15 @@ from __future__ import annotations
 
 import builtins
 import copy
-import json
 import logging
+import math
 import numbers
 import re
 from abc import ABCMeta, abstractmethod
-from pathlib import Path
 from typing import (
     Any,
     Callable,
     Dict,
-    Iterable,
     List,
     Mapping,
     Optional,
@@ -53,7 +51,7 @@ from ..grids.boundaries.axes import BoundariesData
 from ..grids.boundaries.local import BCDataError
 from .cache import cached_method, cached_property
 from .docstrings import fill_in_docstring
-from .misc import Number, classproperty, number, number_array
+from .misc import Number, number, number_array
 from .numba import convert_scalar, jit
 from .typing import NumberOrArray
 
@@ -110,7 +108,7 @@ def _heaviside_implemention(x1, x2=0.5):
         float: 0 if x1 is negative, 1 if x1 is positive, and x2 if x1 == 0
     """
     if np.isnan(x1):
-        return np.nan
+        return math.nan
     elif x1 == 0:
         return x2
     elif x1 < 0:
@@ -146,6 +144,42 @@ class NumpyArrayPrinter(PythonCodePrinter):
         return f"array(broadcast_arrays({arrays}))"
 
 
+def parse_expr_guarded(expression: str, symbols=None, functions=None) -> basic.Basic:
+    """parse an expression using sympy with extra guards
+
+    Args:
+        expression (str):
+            The expression to be parsed
+        symbols:
+            (nested) collection of symbols explicitly treated as symbols by sympy
+        functions:
+            (nested) collection of symbols explicitly treated as functions by sympy
+
+    Returns:
+        :class:`sympy.core.basic.Basic`: The sympy expression
+    """
+    # parse the expression using sympy
+    from sympy.parsing import sympy_parser
+
+    # collect all symbols that are likely present and should thus be interpreted as
+    # symbols by sympy. If we omit defining `local_dict`, many expressions will be
+    # parsed as objects inherent to sympy , which breaks our expressions.
+    local_dict = {}
+
+    def fill_locals(element, sympy_cls):
+        """recursive function for obtaining all symbols"""
+        if isinstance(element, str):
+            local_dict[element] = sympy_cls(element)
+        elif hasattr(element, "__iter__"):
+            for el in element:
+                fill_locals(el, sympy_cls)
+
+    fill_locals(symbols, sympy_cls=sympy.Symbol)
+    fill_locals(functions, sympy_cls=sympy.Function)
+
+    return sympy_parser.parse_expr(expression, local_dict=local_dict)  # type: ignore
+
+
 ExpressionType = Union[float, str, np.ndarray, basic.Basic, "ExpressionBase"]
 
 
@@ -229,52 +263,6 @@ class ExpressionBase(metaclass=ABCMeta):
         else:
             return diff == 0
 
-    @classproperty
-    def _reserved_symbols(self) -> Set[str]:
-        """set: reserved sympy symbols that should not be used in expressions"""
-        try:
-            # try returning a cached version of the list
-            return ExpressionBase._reserved_symbols_cache  # type: ignore
-
-        except AttributeError:
-            # the cache was not present => load list from resources
-            module_path = Path(__file__).resolve().parent
-            resource_path = module_path / "resources" / "reserved_sympy_symbols.json"
-
-            try:
-                with open(resource_path) as f:
-                    ExpressionBase._reserved_symbols_cache = set(json.load(f))  # type: ignore
-            except (IOError, OSError):
-                # cannot read the file, so return a minimal list
-                ExpressionBase._reserved_symbols_cache = {"E", "I", "pi"}  # type: ignore
-
-        return ExpressionBase._reserved_symbols_cache  # type: ignore
-
-    @classmethod
-    def check_reserved_symbols(
-        cls, symbols: Iterable[str], strict: bool = True
-    ) -> None:
-        """throws an error if reserved symbols are found
-
-        Args:
-            symbols (iterable):
-                A sequence or set of strings with symbols to check.
-            strict (bool):
-                Flag determining whether an exception is raised
-        """
-        symbol_set = {s.lower() for s in symbols}
-        reserved_symbols = symbol_set & ScalarExpression._reserved_symbols
-        if any(reserved_symbols):
-            if len(reserved_symbols) == 1:
-                name = reserved_symbols.pop()
-                msg = f"Cannot use reserved symbol `{name}` as field name"
-            else:
-                msg = f"Cannot use reserved symbols {reserved_symbols} as field names"
-            if strict:
-                raise ValueError(msg)
-            else:
-                logging.getLogger(cls.__name__).warning(msg)
-
     @property
     def _free_symbols(self) -> Set:
         """return symbols that appear in the expression and are not in self.consts"""
@@ -315,15 +303,6 @@ class ExpressionBase(metaclass=ABCMeta):
 
         self._logger.debug(f"Expression arguments: {args}")
 
-        # check whether signature contains reserved symbols
-        sig_elements = []
-        for sig in signature:
-            if isinstance(sig, str):
-                sig_elements.append(sig)
-            else:
-                sig_elements.extend(sig)
-        self.check_reserved_symbols(sig_elements, strict=False)
-
         # check whether variables are in signature
         self.vars: Any = []
         found = set()
@@ -521,6 +500,7 @@ class ScalarExpression(ExpressionBase):
         *,
         user_funcs: Optional[Dict[str, Callable]] = None,
         consts: Optional[Dict[str, NumberOrArray]] = None,
+        explicit_symbols: Sequence[str] = None,
         allow_indexed: bool = False,
     ):
         """
@@ -546,6 +526,8 @@ class ScalarExpression(ExpressionBase):
                 A dictionary with user defined constants that can be used in the
                 expression. The values of these constants should either be numbers or
                 :class:`~numpy.ndarray`.
+            explicit_symbols (list of str):
+                List of symbols that need to be interpreted as general sympy symbols
             allow_indexed (bool):
                 Whether to allow indexing of variables. If enabled, array variables are
                 allowed to be indexed using square bracket notation.
@@ -581,11 +563,11 @@ class ScalarExpression(ExpressionBase):
         elif bool(expression):
             # parse expression as a string
             expression = self._prepare_expression(str(expression))
-
-            # parse the expression using sympy
-            from sympy.parsing import sympy_parser
-
-            sympy_expr = sympy_parser.parse_expr(expression)
+            sympy_expr = parse_expr_guarded(
+                expression,
+                symbols=[signature, consts, explicit_symbols],
+                functions=user_funcs,
+            )
 
         else:
             # expression is empty, False or None => set it to zero
@@ -721,6 +703,7 @@ class TensorExpression(ExpressionBase):
         *,
         user_funcs: Optional[Dict[str, Callable]] = None,
         consts: Optional[Dict[str, NumberOrArray]] = None,
+        explicit_symbols: Sequence[str] = None,
     ):
         """
         Warning:
@@ -746,6 +729,8 @@ class TensorExpression(ExpressionBase):
                 A dictionary with user defined constants that can be used in the
                 expression. The values of these constants should either be numbers or
                 :class:`~numpy.ndarray`.
+            explicit_symbols (list of str):
+                List of symbols that need to be interpreted as general sympy symbols
         """
         from sympy.tensor.array.ndim_array import ImmutableNDimArray
 
@@ -768,12 +753,12 @@ class TensorExpression(ExpressionBase):
 
         else:
             # parse expression as a string
-            expression = str(expression)
-
-            # parse the expression using sympy
-            from sympy.parsing import sympy_parser
-
-            sympy_expr = sympy.Array(sympy_parser.parse_expr(expression))
+            sympy_expr_raw = parse_expr_guarded(
+                str(expression),
+                symbols=[signature, consts, explicit_symbols],
+                functions=user_funcs,
+            )
+            sympy_expr = sympy.Array(sympy_expr_raw)
 
         super().__init__(
             expression=sympy_expr,
@@ -1097,6 +1082,8 @@ def evaluate(
     result_data = func(*field_data, None, {}, *extra_args)
 
     # turn result into a proper field
+    if np.isscalar(result_data):
+        result_data = np.broadcast_to(result_data, grid.shape)
     result_rank = result_data.ndim - grid.num_axes
     result_cls = DataFieldBase.get_class_by_rank(result_rank)
     return result_cls(grid, result_data, label=label)
